#ifndef lariat_V1495FRAGMENT_hh
#define lariat_V1495FRAGMENT_hh

#include <iostream>
#include <vector>

class V1495Fragment {

 public:

#pragma pack(push,1)

    typedef struct V1495Header 
    {
      uint32_t fragmentSize;
      uint32_t fragmentType;
      uint32_t nChan;
      uint32_t nTrigPat;
      uint32_t nVetoPat;
      uint32_t M; //!< M of N in coincidence unit
      uint32_t tvMask; //!< trigger and veto mask
    } V1495Header_t ;
    
    typedef struct V1495ChannelData
    {
      char name[16];
      uint32_t num;
      uint32_t counts;
    } V1495ChannelData_t ;

    typedef struct V1495TriggerPatternData
    {
      uint32_t num;
      uint32_t pattern;
      uint32_t counts;
    } V1495TriggerPatternData_t ;
    
#pragma pack(pop)

    V1495Fragment();

    
    // the actual object
    V1495Header_t header;
    std::vector<V1495ChannelData_t> channelData;
    std::vector<V1495TriggerPatternData_t> triggerPatternData;
    std::vector<V1495TriggerPatternData_t> vetoPatternData;

    /**
     * stream the data from this object into a buffer.
     * Returns the number of characters written as a convenience.
     * This function sets fragmentSize and fragmentType.
     **/
    uint32_t streamToBuffer(std::ostream& buffer);

    /**
     * Fill in this object, assuming that the current position
     * in the buffer coincides with the start of this object.
     * Returns the number of characters read as a convenience.
     * Obviously streamToBuffer() and fillFromBuffer() must agree on
     * the structure of the object in the buffer!
     **/
    uint32_t fillFromBuffer(std::istream& buffer);

    
    /**
     * print this object in a user readable format
     **/
    void print(std::ostream& os = std::cout);
    /**
     * Print an html table with containing the data from this fragment.
     * opt=0,1,2 correspond to channels, triggers and vetos
     */
    void printHTML(int opt, std::ostream& os );
};


#endif // lariat_V1495FRAGMENT_hh
