//
//  lariat-online/include/WUTFragment.h    (W.Badgett)
//
//  Description of the  WUT raw data format
//
//
//  The event format is as such:
//  Per 16 microsec time Frame:  
//        One WutFrameHeader struct, 
//        followed by nHits number of WutHit structs
//
//        Each frame timeHeader count is 16 microsec, reset by begin of store
//        Each hit timeBin count is 15.625 ps
//     

#include <iostream>
#include <vector>

#ifndef _WUTFragment_h
#define _WUTFragment_h

class WUTFragment
{
 public:

 enum
 {
   N_THRESHOLDS = 4,
 };

#pragma pack(push,1)

  typedef struct WutFrameHeader
  {
    uint32_t fragmentSize;
    uint32_t fragmentType;
    uint32_t spillCounter;
    uint32_t pollCount;
    uint32_t timeHeader;
    uint32_t nHits;
  } WutFrameHeader_t;

  typedef struct WutHit
  {
    uint8_t  channel;
    uint32_t timeBin;
//    uint32_t tcyTime;
//    uint32_t fineTime;
  } WutHit_t;

#pragma pack(pop)



  // the unpacked object data

  WutFrameHeader_t header;
  std::vector<WutHit_t> hits;

  /**
     * stream the data from this object into a buffer.
     * Returns the number of characters written as a convenience.
     * This function sets fragmentSize and fragmentType.
     **/
  uint32_t streamToBuffer(std::ostream& buffer);
  
  /**
   * Fill in this object, assuming that the current position
   * in the buffer coincides with the start of this object.
   * Returns the number of characters read as a convenience.
   * Obviously streamToBuffer() and fillFromBuffer() must agree on
   * the structure of the object in the buffer!
   **/
  uint32_t fillFromBuffer(std::istream& buffer);

  
  /**
   * print this object in a user readable format
   **/
  void print(std::ostream& os = std::cout);


  static void print(struct WutFrameHeader *header, 
		    std::ostream& os = std::cout);
  static void printHit(int h, struct WutHit *hit, 
		       std::ostream& os = std::cout);


};

#endif
