////////////////////////////////////////////////////////////////////////
// Class:       SpillTrailerDump
// Module Type: analyzer
// File:        SpillTrailerDump_module.cc
// Description: Prints out information about each event.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"

#include "art/Utilities/Exception.h"
#include "ModuleUtils.hh"
#include "artdaq-core/Data/Fragments.hh"
#include "include/LariatFragment.h"

#include <algorithm>
#include <cassert>
#include <cmath>
#include <fstream>
#include <iomanip>
#include <vector>
#include <iostream>

namespace lariat {
  class SpillTrailerDump;
}

class lariat::SpillTrailerDump : public art::EDAnalyzer 
{
public:
  explicit SpillTrailerDump(fhicl::ParameterSet const & pset);
  virtual ~SpillTrailerDump() {}

  virtual void analyze(art::Event const & evt);

private:
  std::string raw_data_label_;
};


lariat::SpillTrailerDump::SpillTrailerDump(fhicl::ParameterSet const & pset)
    : EDAnalyzer(pset),
      raw_data_label_(pset.get<std::string>("raw_data_label"))
{
}

void lariat::SpillTrailerDump::analyze(art::Event const & evt)
{
  art::EventNumber_t eventNumber = evt.event();
  std::cout << "Event " << eventNumber << std::endl;
  art::Handle<artdaq::Fragments> raw;
  evt.getByLabel(raw_data_label_, "SPILL", raw);
  if ( raw.isValid() ) 
  {
    if (raw->size() != 1 ) 
    {
      throw cet::exception("Error in SpillTrailerDump module: expected one and only one spill fragment");
    }
    
    const auto& frag((*raw)[0]);
    LariatFragment data((const char *)frag.dataBeginBytes(),frag.dataSizeBytes());
    data.print(std::cout);
    data.printSpillTrailer();
  } 
  else 
  {
    throw cet::exception("Error in SpillTrailerDump_module: expected one and only one spill fragment");
  }
  std::cout << std::endl;
}

DEFINE_ART_MODULE(lariat::SpillTrailerDump)
