////////////////////////////////////////////////////////////////////////
// Class:       V1495Dump
// Module Type: analyzer
// File:        V1495Dump_module.cc
// Description: Prints out information about each event.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"

#include "art/Utilities/Exception.h"
#include "lariat-artdaq/Overlays/V1495HitFragment.hh"
#include "ModuleUtils.hh"
#include "artdaq-core/Data/Fragments.hh"
#include "include/WUTFragment.h"

#include <algorithm>
#include <cassert>
#include <cmath>
#include <fstream>
#include <iomanip>
#include <vector>
#include <iostream>

namespace lariat {
  class V1495Dump;
}

class lariat::V1495Dump : public art::EDAnalyzer {
public:
  explicit V1495Dump(fhicl::ParameterSet const & pset);
  virtual ~V1495Dump() {}

  virtual void analyze(art::Event const & evt);

private:
  std::string raw_data_label_;
  uint32_t max_wuts_to_show_;
};


lariat::V1495Dump::V1495Dump(fhicl::ParameterSet const & pset)
    : EDAnalyzer(pset),
      raw_data_label_(pset.get<std::string>("raw_data_label"))
{
}

void lariat::V1495Dump::analyze(art::Event const & evt)
{

  art::EventNumber_t eventNumber = evt.event();

  art::Handle<artdaq::Fragments> raw;
  evt.getByLabel(raw_data_label_, "SPILL", raw);

  if ( raw.isValid() ) {

    if (raw->size() != 1 ) {
      throw cet::exception("Error in V1495Dump module: expected one and only one spill fragment");
    }
    
    const auto& frag((*raw)[0]);

    std::vector<const uint8_t*> lariatFragPtrs;

    LariatFragment::LariatFragmentType lariatFragType = LariatFragment::LariatFragmentType::FRAGMENT_TYPE_V1495;

    getLariatFragments(frag, 
     		       lariatFragType,
		       lariatFragPtrs
		       );

    std::cout << std::endl;
    std::cout << "Run " << evt.run() << ", subrun " << evt.subRun()
       	      << ", spill " << eventNumber << " has " << lariatFragPtrs.size()
   	      << " fragment(s) of type " << lariatFragType << std::endl;
      
    for (size_t idx = 0; idx < lariatFragPtrs.size(); ++idx) {

      lariat::V1495HitFragment bb( lariatFragPtrs[idx]  );

      bb.printHeader();

      std::cout << std::endl;
      std::cout << "######################################################################" << std::endl;
      std::cout << std::endl;
      std::cout << std::endl;
    }
  } else {
    throw cet::exception("Error in V1495Dump_module: expected one and only one spill fragment");
  }

  std::cout << std::endl;

}

DEFINE_ART_MODULE(lariat::V1495Dump)
