#include "lariat-artdaq/Overlays/FragmentType.hh"

#include <algorithm>
#include <cassert>
#include <string>
#include <vector>

// Note the "names" vector has the empty string in position 5
// (counting up from 1), to reflect the lack of a Lariat fragment type
// value 5

namespace {
  static std::vector<std::string> const
  names { "RUNINFO", "SPILLINFO", "CAEN_ADC", "V1495", "", "TDC_CONTROLLER_HEADER", "TDC_TDC_SPILL_HEADER", "TDC_TDC_EVENT_HEADER", "TOY1", "TOY2", "MISSED", "SPILL", "UNKNOWN" };
}

lariat::FragmentType
lariat::toFragmentType(std::string t_string)
{
  std::transform(t_string.begin(),
                 t_string.end(),
                 t_string.begin(),
                 toupper);
  auto it = std::find(names.begin(), names.end(), t_string);
  return (it == names.end()) ?
    FragmentType::INVALID :
    static_cast<FragmentType>(1 + (it - names.begin()));
}

std::string
lariat::fragmentTypeToString(FragmentType val)
{
  if (val < FragmentType::INVALID) {
    return names[val - 1];
  }
  else {
    return "INVALID/UNKNOWN";
  }
}
