#ifndef lariat_artdaq_Overlays_FragmentType_hh
#define lariat_artdaq_Overlays_FragmentType_hh
#include "artdaq-core/Data/Fragment.hh"

// John F., 3/10/14 -- TOY1 and TOY2 fragment values have been changed
// from being 2 and 3, respectively, to 9 and 10. This is because
// Lariat has its own fragment numbering system whereby the spillinfo
// and CAEN adc fragments type values are 2 and 3; for full details
// see
// https://cdcvs.fnal.gov/redmine/projects/lariat-online/wiki/Raw_Data_Format_and_Readout_Source_code

// Please also note that unlike in artdaq-demo, the "MISSED" fragment
// is the second-to-last, rather than the first. This is so the lariat
// fragment type of "1" corresponding to run info can be preserved

namespace lariat {

  namespace detail {
    enum FragmentType : artdaq::Fragment::type_t
    { 	RUNINFO = 1,
	SPILLINFO,
	CAEN_ADC,
	V1495,
	TDC_CONTROLLER_HEADER = 6,
	TDC_TDC_SPILL_HEADER,
	TDC_TDC_EVENT_HEADER,
	TOY1,
	TOY2,
	MISSED,
	SPILL,
        INVALID // Should always be last.
        };

    // Safety check.
    static_assert(artdaq::Fragment::isUserFragmentType(FragmentType::INVALID - 1),
                  "Too many user-defined fragments!");
  }

  using detail::FragmentType;

  FragmentType toFragmentType(std::string t_string);
  std::string fragmentTypeToString(FragmentType val);
}
#endif /* lariat_artdaq_Overlays_FragmentType_hh */
