#include "CAENFragment.h"
#include "LariatFragment.h"
#include <stdexcept>
#include <string>
#include <vector>

uint32_t CAENFragment::streamToBuffer(std::ostream& buffer){
  

  uint32_t nchar_written=0;
    
  header.fragmentType = LariatFragment::FRAGMENT_TYPE_CAEN_ADC;

  using std::streamsize;
  const streamsize headersize=sizeof(CaenHeader_t);

  
  header.fragmentSize = headersize + sizeof(WaveFormData_t::value_type)*header.nChannels*header.nSamples;
  
  // write the header
  
  buffer.write(reinterpret_cast<const char*>(&header),headersize);
  nchar_written+=headersize;
  
  /////// write the data from each digitizer channel////////
  if(header.nChannels!=waveForms.size()) {
    throw std::length_error("CAENFragment::streamToBuffer : header.nChannels!=waveForms.size()");
  }
  
  for(size_t i=0; i<header.nChannels; ++i){

    const WaveForm& wave=waveForms[i];
    
    if(wave.data.size()!=header.nSamples){
      throw std::length_error("CAENFragment::streamToBuffer : header.nSamples!=wave.data.size()");
    }
    // compute the size of the waveform data in bytes
    const streamsize wavesize=wave.data.size()*sizeof(WaveFormData_t::value_type);
    // now write the waveform data
    // by ISO standard the elements of a vector are required to be contiguous
    // so, this ought to work!
    buffer.write(reinterpret_cast<const char*>(& (wave.data)[0]),wavesize);
    nchar_written+=wavesize;
  }
  return nchar_written;

}
  

uint32_t CAENFragment::fillFromBuffer(std::istream& buffer){

  uint32_t nchar_read=0;
  
  using std::streamsize;
  const streamsize headersize=sizeof(CaenHeader_t);  

  // read the header
  buffer.read(reinterpret_cast<char*>(&header),headersize);
  nchar_read+=headersize;
  
  // read the channel data
  for(size_t i=0; i<header.nChannels; ++i){
    WaveForm wave;
    // compute the size of the waveform data in bytes
    const streamsize wavesize=header.nSamples*sizeof(WaveFormData_t::value_type);    
    // create a block of memory 
    WaveFormData_t::value_type* wave_array = new WaveFormData_t::value_type[header.nSamples]; 

    // read into the memory block
    buffer.read(reinterpret_cast<char*>(wave_array),wavesize);
    nchar_read+=wavesize;
    
    // push the array data into the vector inside of wave
    wave.data.assign(wave_array, wave_array+header.nSamples);    
    waveForms.push_back(wave);
    delete wave_array;
  }

  return nchar_read;
}

  
void CAENFragment::print(std::ostream& os){
  
  os<<"fragmentSize = "<<header.fragmentSize <<std::endl;
  os<<"fragmentType = "<<header.fragmentType <<std::endl;
  os<<"eventCounter = "<<header.eventCounter <<std::endl;
  os<<"eventSize = "<<header.eventSize<<std::endl;
  os<<"boardId = "<<header.boardId <<std::endl;
  os<<"pattern = "<<header. pattern<<std::endl;
  os<<"triggerTimeTag = "<<header. triggerTimeTag<<std::endl;
  os<<"channelMask = "<<header.channelMask <<std::endl;
  os<<"nChannels = "<<header. nChannels <<std::endl;
  os<<"nSamples = "<<header. nSamples<<std::endl;
  os<<"timeStampSec = "<<header. timeStampSec <<std::endl;
  os<<"timeStampNSec = "<<header. timeStampNSec<<std::endl;
  

  
}

void CAENFragment::printWaves(int* chans, int nchan, std::ostream& os){
  std::vector<int> all_chans(header.nChannels, 0);
  for(size_t i=0; i<header.nChannels; ++i) all_chans[i]=i;

  if(chans==0) {chans = all_chans.data(); nchan=header.nChannels;}
  
  // print header
  os<<"    ";
  for(int i=0; i<nchan; i++){
    int channel=chans[i];
    os.width(1); os.precision(1);
    os<<"|";
    os.width(4); os.precision(4);
    os<<std::dec<<channel;
  }
  os<<'\n';

  os.width(1);
  os<<"-----";
  for(int i=0; i<nchan; i++){
    os<<"-----";
  }
  os<<'\n';

  for(size_t is=0; is<header.nSamples; ++is){
    os.width(4); os.precision(4); os<<std::dec<<is;
    for(int ic=0; ic<nchan; ic++){
      int channel=chans[ic];
      os.width(1); os.precision(1);
      os<<"|";
      os.width(4); os.precision(4);
      os<<std::hex<<waveForms[channel].data[is];
    }
    os<<'\n';
  }
  
  

}
