//
//  lariat-online/daq/src/LariatFragment.cc
//

#include "LariatFragment.h"
#include "RawBuffer.h"
#include <fstream>

// My three constructors:

LariatFragment::LariatFragment(std::istream & in)
{
  fillFromBuffer(in);
}

LariatFragment::LariatFragment(std::string & fileName)
{
  std::ifstream in(fileName);
  fillFromBuffer(in);
  in.close();
}

LariatFragment::LariatFragment(const char * buffer, size_t size)
{
  char * ptr = (char *)buffer;
  RawBuffer rawBuffer(ptr,size);
  std::istream in(&rawBuffer);
  fillFromBuffer(in);
}


//
// Convenience function:
//

const char* LariatFragment::fragmentNameFromType(int id)
{
  switch(id){
  case FRAGMENT_TYPE_RUNINFO:
    return "Run Info";
  case FRAGMENT_TYPE_SPILLINFO:
    return "Spill Info";
  case FRAGMENT_TYPE_SPILLTRAILER:
    return "Spill Trailer";
  case FRAGMENT_TYPE_CAEN_ADC:
    return "CAEN Digitizer";
  case FRAGMENT_TYPE_V1495:
    return "CAEN V1495";
  case FRAGMENT_TYPE_TDC_CONTROLLER_HEADER:
    return "TDC Controller Header";
  case FRAGMENT_TYPE_TDC_TDC_SPILL_HEADER:
    return "TDC Spill Header";
  case FRAGMENT_TYPE_TDC_TDC_EVENT_HEADER:
    return "TDC Event Header";
  case FRAGMENT_TYPE_WUT_HIT_HEADER:
    return "WUT Hit Header";
  case FRAGMENT_TYPE_LARIAT_CONFIG:
    return "Lariat Config";
  default:
    return "Unknown";
    break;
  }
  return "Unknown";
}


/*
uint32_t LariatFragment::streamToBuffer(std::ostream& output){

  //

  return 0;
}
*/

// Upacking functions:
  
uint32_t LariatFragment::fillFromBuffer(std::istream& input)
{
  using std::cout; using std::endl;
  int fragmentSize = 0;
  int fragmentType = 0;

  nBytes=0;
  for ( int i=0; i<FRAGMENT_TYPE_N; i++) { count[i] = 0;}

  while ( input.good() )
  {
    std::streampos starting_position=input.tellg();
    input.read((char *)&fragmentSize,sizeof(fragmentSize));
    input.read((char *)&fragmentType,sizeof(fragmentType));
    if ( input.good() )
    {
      if ( fragmentType < FRAGMENT_TYPE_N ) { count[fragmentType]++;}
      input.seekg(starting_position);

      // Case statement doesn't work here due to scoping
      if( fragmentType ==  FRAGMENT_TYPE_RUNINFO)
      {
	nBytes += fillRunInfoFromBuffer(input);
      }
      else if( fragmentType ==  FRAGMENT_TYPE_SPILLINFO)
      {
	nBytes += fillSpillInfoFromBuffer(input);
      }
      else if( fragmentType ==  FRAGMENT_TYPE_SPILLTRAILER)
      {
	nBytes += fillSpillTrailerFromBuffer(input);
	printSpillTrailer();
      }
      else if( fragmentType ==  FRAGMENT_TYPE_CAEN_ADC)
      {
	CAENFragment frag;
	nBytes+=frag.fillFromBuffer(input);
	caenFrags.push_back(frag);      
      }
      else if(fragmentType ==  FRAGMENT_TYPE_WUT_HIT_HEADER)
      {
      	WUTFragment frag;
      	nBytes+=frag.fillFromBuffer(input);
      	wutFrags.push_back(frag);	
      }
      else if( fragmentType ==  FRAGMENT_TYPE_V1495)
      {
	V1495Fragment frag;
	nBytes+=frag.fillFromBuffer(input);
	v1495Frags.push_back(frag);      
      }
      else if( fragmentType ==  FRAGMENT_TYPE_TDC_CONTROLLER_HEADER )
      {
	TDCFragment frag;
	nBytes += frag.fillFromBuffer(input);
	tdcFrags.push_back(frag);      
      }
      else if( fragmentType ==  FRAGMENT_TYPE_LARIAT_CONFIG )
      {
	nBytes += fillLariatConfigFromBuffer(input);
      }
      else
      {
	std::cout << "Unrecognized fragmentType " << fragmentType << 
	  " 0x" << std::hex << fragmentType << std::dec 
		  << " of size " << fragmentSize << std::endl;
	std::streamoff fSize = fragmentSize;
	
	// Fast forward to next fragment:
	input.seekg(starting_position + fSize);
	nBytes += fragmentSize;
      }
    }
  }

  for ( int i=0; i<FRAGMENT_TYPE_N; i++)
  { std::cout << "Type " << i << " had " << count[i] << " fragments" << std::endl;}

  return nBytes;
}

uint32_t LariatFragment::fillRunInfoFromBuffer(std::istream& input)
{
  uint32_t nBytes = 0;
  input.read((char *)&runInfo,sizeof(struct RunInfo));
  nBytes += sizeof(struct RunInfo);
  return(nBytes);
}

uint32_t LariatFragment::fillSpillInfoFromBuffer(std::istream& input)
{
  uint32_t nBytes = 0;
  input.read((char *)&spillInfo,sizeof(struct SpillInfo));
  nBytes += sizeof(struct SpillInfo);
  return(nBytes);
}

uint32_t LariatFragment::fillSpillTrailerFromBuffer(std::istream& input)
{
  uint32_t nBytes = 0;
  input.read((char *)&spillTrailer,sizeof(struct SpillTrailer));
  nBytes += sizeof(struct SpillTrailer);
  return(nBytes);
}

uint32_t LariatFragment::fillLariatConfigFromBuffer(std::istream& input)
{
  uint32_t nBytes = 0;
  input.read((char *)&lariatConfig.fragmentSize, 
	     sizeof(lariatConfig.fragmentSize));
  nBytes += sizeof(lariatConfig.fragmentSize);

  input.read((char *)&lariatConfig.fragmentType, 
	     sizeof(lariatConfig.fragmentType));
  nBytes += sizeof(lariatConfig.fragmentSize);

  uint32_t remainder = lariatConfig.fragmentSize - nBytes;
  lariatConfig.xml = (char *)malloc(remainder);
  input.read(lariatConfig.xml, remainder);

  return(lariatConfig.fragmentSize);
}

void LariatFragment::printLariatConfig(std::ostream & os)
{
  os << "LariatConfig"       << std::endl;
  os << "  FragmentSize    " << lariatConfig.fragmentSize  << std::endl;
  os << "  FragmentType    " << lariatConfig.fragmentType  << std::endl;
  os << "  XML:" << std::endl;
  os << lariatConfig.xml << std::endl;
}

void LariatFragment::printSpillTrailer(std::ostream & os)
{
  os << "SpillTrailer"       << std::endl;
  os << "  FragmentSize    " << spillTrailer.fragmentSize  << std::endl;
  os << "  FragmentType    " << spillTrailer.fragmentType  << std::endl;
  os << "  RunNumber       " << spillTrailer.runNumber     << std::endl;
  os << "  SpillNumber     " << spillTrailer.spillNumber   << std::endl;
  os << "  TimeStamp       " << spillTrailer.timeStamp     << std::endl;
  os.precision(15);
  os << "  SpillSnapshot   " << spillTrailer.spillSnapshot << std::endl;
  os << "  SpillOffset     " << spillTrailer.spillOffset   << std::endl;
  os << "  SpillV1740      " << spillTrailer.spillV1740    << std::endl;
  os << "  SpillV1751      " << spillTrailer.spillV1751    << std::endl;
  os << "  CountV1740      " << spillTrailer.countV1740    << std::endl;
  os << "  CountV1751      " << spillTrailer.countV1751    << std::endl;
}

void LariatFragment::print(std::ostream & os)
{
  os << "nBytes Read " << nBytes << std::endl;
  for ( int i=0; i<FRAGMENT_TYPE_N; i++)
  {
    os << i << ": " <<fragmentNameFromType(i) << " " << count[i] << std::endl;
  }
}

void LariatFragment::checkOStream(std::ostream & buffer)
{
  if ( buffer.bad() || buffer.eof() || buffer.fail() || !buffer.good() )
  {
    std::cout << "Buffer error: ";
    if ( buffer.bad() )  std::cout << "BAD ";
    if ( buffer.eof() )  std::cout << "EOF ";
    if ( buffer.fail() ) std::cout << "FAIL ";
    if ( !buffer.good() ) std::cout << "NOT_GOOD ";
    throw std::ios_base::failure("Output Stream Failure");
  }
  return;
}
