//
//   lariat-online/daq/src/TDCFragment   (W.Badgett)
//

#include "TDCFragment.h"
#include <stdexcept>

uint32_t TDCFragment::streamToBuffer(std::ostream& buffer)
{
  return(0);
}

uint32_t TDCFragment::fillFromBuffer(std::istream& buffer)
{
  uint32_t bytes = 0;

  buffer.read(reinterpret_cast<char*>(&controllerHeader),
	      sizeof(struct TdcControllerHeader));
  if ( buffer.eof() ) { return(bytes);}

  bytes += sizeof(struct TdcControllerHeader);
  //  printTdcControllerHeader(controllerHeader);

  for ( size_t i=0; i<controllerHeader.nTDCs; ++i)
  {
    TdcSpillHeader tdcSpillHeader;
    buffer.read(reinterpret_cast<char*>(&tdcSpillHeader), sizeof(struct TdcSpillHeader));
    if ( buffer.eof() ) { return(bytes);}

    bytes += sizeof(struct TdcSpillHeader);
    if ( tdcSpillHeader.spillTriggerCounter != 
	 controllerHeader.triggerCounts )
    {
      char text[1024];
      sprintf(text,"Fatal error: TDC has %d triggers, controller has %d",
	      tdcSpillHeader.spillTriggerCounter,
	      controllerHeader.triggerCounts );
      // throw(std::length_error(text));
    }
    //    printTdcSpillHeader(tdcSpillHeader);
    tdcSpillHeaders.push_back(tdcSpillHeader);
  }

  for ( size_t i=0; i<controllerHeader.triggerCounts; ++i)
  {
    std::vector<TdcEventData> tdcEventVector;
    for ( size_t j=0; j<controllerHeader.nTDCs; ++j)
    {
      TdcEventData tdcEventData;
      buffer.read(reinterpret_cast<char*>(&tdcEventData.tdcEventHeader), 
		  sizeof(struct TdcEventHeader));
      if ( buffer.eof() ) { return(bytes);}
      bytes += sizeof(struct TdcEventHeader);
      //printTdcEventHeader(tdcEventData.tdcEventHeader);
      if ( tdcEventData.tdcEventHeader.tdcNumber != (char)(j+1))
      {
	std::cout << "**** Fatal mismatch " << (int)tdcEventData.tdcEventHeader.tdcNumber
		  << " != " << (j+1) << std::endl;
      }

      for ( size_t k=0; k<tdcEventData.tdcEventHeader.nHits; ++k)
      {
	struct TdcHit tdcHit;
	buffer.read(reinterpret_cast<char*>(&tdcHit), sizeof(struct TdcHit));
	if ( buffer.eof() ) { return(bytes);}
	bytes += sizeof(struct TdcHit);

	tdcEventData.tdcHits.push_back(tdcHit);
      }
      tdcEventVector.push_back(tdcEventData);
    }
    tdcEvents.push_back(tdcEventVector);
  }
  return(bytes);
}

void TDCFragment::printTdcControllerHeader(TdcControllerHeader header,
					   std::ostream & os)
{
  os << "TdcControllerHeader" << std::endl;
  os << "  FragmentSize         " << header.fragmentSize<< std::endl;
  os << "  FragmentType         " << header.fragmentType<< std::endl;
  os << "  SpillWordCount       " << header.spillWordCount<< std::endl;
  os << "  SpillWordCountBuffer " << header.spillWordCountBuffer<< std::endl;
  os << "  SpillCounter         " << header.spillCounter<< std::endl;
  os << "  ControllerNumber     " << (int)header.controllerNumber<< std::endl;
  os << "  Year                 " << (int)header.year<< std::endl;
  os << "  Month                " << (int)header.month<< std::endl;
  os << "  Day                  " << (int)header.day<< std::endl;
  os << "  Hours                " << (int)header.hours<< std::endl;
  os << "  Minutes              " << (int)header.minutes<< std::endl;
  os << "  Seconds              " << (int)header.seconds<< std::endl;
  os << "  TriggerCounts        " << header.triggerCounts<< std::endl;
  os << "  TdcStatus            0x" << std::hex << header.tdcStatus
     << std::endl;
  os << "  LinkStatus           0x" << std::hex << header.linkStatus 
     << std::endl;
  os << "  TdcActiveMask        0x" << std::hex << header.tdcActiveMask 
     << std::endl;
  os << "  nTDCs                " << std::dec << header.nTDCs<< std::endl;
}

void TDCFragment::print(std::ostream & os)
{
  printTdcControllerHeader(controllerHeader,os);
  os << "  tdcSpillHeader.size()" << tdcSpillHeaders.size() << std::endl;
  os << "  tdcEvents.size()     " << tdcEvents.size() << std::endl;
  for ( auto &i: tdcSpillHeaders )
  {
    printTdcSpillHeader(i,os);
  }

  // Loop over events/triggers first, one entry per event
  for (auto &i: tdcEvents )
  {
    // Here "i" is std::vector<TdcEventData> which should have one entry per TDC
    // Next loop over TDCs within the event
    for ( auto &j: i )
    {
      // Now "j" is TdcEventData class

      printTdcEventHeader(j.tdcEventHeader,os);
      int h=0;
      for ( auto &k: j.tdcHits )
      {
	printTdcHit(k,h,os);
	h++;
      }
    }
  }
}

 void TDCFragment::printTdcSpillHeader(TdcSpillHeader header,
				       std::ostream & os)
{
  os << "TdcSpillHeader" << std::endl;
  os << "  FragmentSize         " << header.fragmentSize<< std::endl;
  os << "  FragmentType         " << header.fragmentType<< std::endl;
  os << "  SpillWordCount       " << header.spillWordCount<< std::endl;
  os << "  SpillTriggerCounter  " << header.spillTriggerCounter<< std::endl;
  os << "  SpillStatus          0x" << std::hex << header.spillStatus << std::endl;
  os << "  ControllerNumber     " << std::dec << (int)header.controllerNumber<< std::endl;
  os << "  TdcNumber            " << (int)header.tdcNumber<< std::endl;
}

void TDCFragment::printTdcEventHeader(TdcEventHeader header,
				      std::ostream & os)
{
  os << "TdcEventHeader" << std::endl;
  os << "  FragmentSize         " << header.fragmentSize<< std::endl;
  os << "  FragmentType         " << header.fragmentType<< std::endl;
  os << "  EventWordCount       " << (int)header.eventWordCount<< std::endl;
  os << "  ControllerNumber     " << (int)header.controllerNumber<< std::endl;
  os << "  TdcNumber            " << (int)header.tdcNumber<< std::endl;
  os << "  EventStatus          " << (int)header.eventStatus<< std::endl;
  os << "  TriggerCounter       " << header.triggerCounter<< std::endl;
  os << "  TriggerType          " << (int)header.triggerType<< std::endl;
  os << "  ControllerTimeStamp  " << header.controllerTimeStamp<< std::endl;
  os << "  TdcTimeStamp         " << header.tdcTimeStamp<< std::endl;
  os << "  nHits                " << header.nHits<< std::endl;
}

void TDCFragment::printTdcHit(TdcHit hit, uint32_t index,
			      std::ostream & os)
{
  char text[256];
  sprintf(text,"  Hit %3d  Channel %5d   TimeBin %5d\n", index,
	 hit.channel, hit.timeBin);
  os << text;
}
