#include "WUTFragment.h"
#include "LariatFragment.h"
#include <stdexcept>


uint32_t WUTFragment::streamToBuffer(std::ostream& buffer){

  uint32_t nchar_written=0;
    
  header.fragmentType = LariatFragment::FRAGMENT_TYPE_WUT_HIT_HEADER;

  using std::streamsize;
  const streamsize headersize=sizeof(WutFrameHeader_t);
  const streamsize hitsize = sizeof(WutHit_t);
  
  header.fragmentSize = headersize + hitsize*header.nHits;

  // write the header

  buffer.write(reinterpret_cast<const char*>(&header),headersize);
  nchar_written+=headersize;
  
  // write the channel data
  if(header.nHits!=hits.size()) {
    throw std::length_error("WUTFragment::streamToBuffer : header.nHits!=hits.size()");
  }
  for(size_t i=0; i<header.nHits; ++i){
    WutHit_t data=hits[i];
    buffer.write(reinterpret_cast<const char*>(&data),hitsize);
    nchar_written+=hitsize;
  }
  return nchar_written;

}


uint32_t WUTFragment::fillFromBuffer(std::istream& buffer){

  uint32_t nchar_read=0;
  
  using std::streamsize;
  const streamsize headersize=sizeof(WutFrameHeader_t);
  const streamsize hitsize = sizeof(WutHit_t);
 
  
  //read the header

  buffer.read(reinterpret_cast<char*>(&header),headersize);
  nchar_read+=headersize;

  //read the channel data

  for(size_t i=0; i<header.nHits; ++i)
  {
    WutHit_t data;
    buffer.read(reinterpret_cast<char*>(&data),hitsize);
    nchar_read+=hitsize;
    hits.push_back(data);
  }
  if(header.nHits!=hits.size()) {
    throw std::length_error("WUTFragment::fillFromBuffer : header.nhits!=hits.size()");
  }  

  return nchar_read;
}

void WUTFragment::print(std::ostream& os){
  print( &(header),os);
  for(size_t ihit=0; ihit<header.nHits; ++ihit) printHit(ihit, &(hits[ihit]),os);
}

void WUTFragment::print(struct WutFrameHeader *header, std::ostream& os){
 os << "WutHitFrame" << '\n';
    os << "  FragmentSize         " << header->fragmentSize << '\n';
    os << "  FragmentType         " << header->fragmentType << '\n';
    os << "  SpillCounter         " << header->spillCounter << '\n';
    os << "  PollCount            " << header->pollCount    << '\n';
    char buf[256];
    
    snprintf(buf,256,"  TimeHeader         0x%8.8X (%d)", header->timeHeader, 
	     static_cast<int>(header->timeHeader) );
    os<<buf<<'\n';
    os << "  nHits                " << header->nHits << std::endl;
}

void WUTFragment::printHit(int h, struct WutHit *hit, std::ostream& os)
{
    char buf[256];
    snprintf(buf,256,"  Hit %4d  Channel 0x%X (%2d)    TimeBin  0x%8.8X (%u)",
	     h,
	     hit->channel, hit->channel,
	     hit->timeBin, hit->timeBin);
    os<<buf<<std::endl;
}

